@@ -15,6 +15,7 @@ use crate::Private;
 use crate::PropertyAttribute;
 use crate::Value;
 use std::convert::TryFrom;
+use std::num::NonZeroI32;
 
 extern "C" {
   fn v8__Object__New(isolate: *mut Isolate) -> *const Object;
@@ -379,11 +380,13 @@ impl Object {
     .into()
   }
 
-  /// The `Object` specific equivalent of `Data::get_hash()`.
-  /// This function is kept around for testing purposes only.
-  #[doc(hidden)]
-  pub fn get_identity_hash(&self) -> int {
-    unsafe { v8__Object__GetIdentityHash(self) }
+  /// Returns the V8 hash value for this value. The current implementation
+  /// uses a hidden property to store the identity hash.
+  ///
+  /// The return value will never be 0. Also, it is not guaranteed to be
+  /// unique.
+  pub fn get_identity_hash(&self) -> NonZeroI32 {
+    unsafe { NonZeroI32::new_unchecked(v8__Object__GetIdentityHash(self)) }
   }
 
   /// Returns the context in which the object was created.
