@@ -50,6 +50,13 @@ class V8_BASE_EXPORT EmulatedVirtualAddressSubspace final
 
   bool FreePages(Address address, size_t size) override;
 
+  Address AllocateSharedPages(Address hint, size_t size,
+                              PagePermissions permissions,
+                              PlatformSharedMemoryHandle handle,
+                              uint64_t offset) override;
+
+  bool FreeSharedPages(Address address, size_t size) override;
+
   bool SetPagePermissions(Address address, size_t size,
                           PagePermissions permissions) override;
 
@@ -61,7 +68,7 @@ class V8_BASE_EXPORT EmulatedVirtualAddressSubspace final
 
   std::unique_ptr<v8::VirtualAddressSpace> AllocateSubspace(
       Address hint, size_t size, size_t alignment,
-      PagePermissions max_permissions) override;
+      PagePermissions max_page_permissions) override;
 
   bool DiscardSystemPages(Address address, size_t size) override;
 
@@ -92,6 +99,13 @@ class V8_BASE_EXPORT EmulatedVirtualAddressSubspace final
     return Contains(unmapped_base(), unmapped_size(), addr, length);
   }
 
+  // Helper function to define a limit for the size of allocations in the
+  // unmapped region. This limit makes it possible to estimate the expected
+  // runtime of some loops in the Allocate methods.
+  bool IsUsableSizeForUnmappedRegion(size_t size) const {
+    return size <= (unmapped_size() / 2);
+  }
+
   // Size of the mapped region located at the beginning of this address space.
   const size_t mapped_size_;
 
