@@ -19,318 +19,6 @@ namespace v8 {
 namespace internal {
 namespace compiler {
 
-class GraphAssembler::BasicBlockUpdater {
- public:
-  BasicBlockUpdater(Schedule* schedule, Graph* graph,
-                    CommonOperatorBuilder* common, Zone* temp_zone);
-
-  Node* AddNode(Node* node);
-  Node* AddNode(Node* node, BasicBlock* to);
-  Node* AddClonedNode(Node* node);
-
-  BasicBlock* NewBasicBlock(bool deferred);
-  BasicBlock* SplitBasicBlock();
-  void AddBind(BasicBlock* block);
-  void AddBranch(Node* branch, BasicBlock* tblock, BasicBlock* fblock);
-  void AddGoto(BasicBlock* to);
-  void AddGoto(BasicBlock* from, BasicBlock* to);
-  void AddTailCall(Node* node);
-
-  void StartBlock(BasicBlock* block);
-  BasicBlock* Finalize(BasicBlock* original);
-
-  BasicBlock* original_block() { return original_block_; }
-  BasicBlock::Control original_control() { return original_control_; }
-  Node* original_control_input() { return original_control_input_; }
-
- private:
-  enum State { kUnchanged, kChanged };
-
-  Zone* temp_zone() { return temp_zone_; }
-
-  bool IsOriginalNode(Node* node);
-  void UpdateSuccessors(BasicBlock* block);
-  void SetBlockDeferredFromPredecessors();
-  void RemoveSuccessorsFromSchedule();
-  void CopyForChange();
-
-  Zone* temp_zone_;
-
-  // Current basic block we are scheduling.
-  BasicBlock* current_block_;
-
-  // The original block that we are lowering.
-  BasicBlock* original_block_;
-
-  // Position in the current block, only applicable in the 'unchanged' state.
-  BasicBlock::iterator node_it_;
-  BasicBlock::iterator end_it_;
-
-  Schedule* schedule_;
-  Graph* graph_;
-  CommonOperatorBuilder* common_;
-
-  // The nodes in the original block if we are in 'changed' state. Retained to
-  // avoid invalidating iterators that are iterating over the original nodes of
-  // the block.
-  NodeVector saved_nodes_;
-
-  // The original control, control input and successors, to enable recovery of
-  // them when we finalize the block.
-  struct SuccessorInfo {
-    BasicBlock* block;
-    size_t index;
-  };
-  ZoneVector<SuccessorInfo> saved_successors_;
-  BasicBlock::Control original_control_;
-  Node* original_control_input_;
-  bool original_deferred_;
-  size_t original_node_count_;
-
-  State state_;
-};
-
-GraphAssembler::BasicBlockUpdater::BasicBlockUpdater(
-    Schedule* schedule, Graph* graph, CommonOperatorBuilder* common,
-    Zone* temp_zone)
-    : temp_zone_(temp_zone),
-      current_block_(nullptr),
-      original_block_(nullptr),
-      schedule_(schedule),
-      graph_(graph),
-      common_(common),
-      saved_nodes_(schedule->zone()),
-      saved_successors_(schedule->zone()),
-      original_control_(BasicBlock::kNone),
-      original_control_input_(nullptr),
-      original_deferred_(false),
-      original_node_count_(graph->NodeCount()),
-      state_(kUnchanged) {}
-
-Node* GraphAssembler::BasicBlockUpdater::AddNode(Node* node) {
-  return AddNode(node, current_block_);
-}
-
-Node* GraphAssembler::BasicBlockUpdater::AddNode(Node* node, BasicBlock* to) {
-  if (state_ == kUnchanged) {
-    DCHECK_EQ(to, original_block());
-
-    if (node_it_ != end_it_ && *node_it_ == node) {
-      node_it_++;
-      return node;
-    }
-
-    CopyForChange();
-  }
-
-  // Add the node to the basic block.
-  DCHECK(!schedule_->IsScheduled(node));
-  schedule_->AddNode(to, node);
-  return node;
-}
-
-Node* GraphAssembler::BasicBlockUpdater::AddClonedNode(Node* node) {
-  DCHECK(node->op()->HasProperty(Operator::kPure));
-  if (state_ == kUnchanged) {
-    CopyForChange();
-  }
-
-  if (schedule_->IsScheduled(node) &&
-      schedule_->block(node) == current_block_) {
-    // Node is already scheduled for the current block, don't add it again.
-    return node;
-  } else if (!schedule_->IsScheduled(node) && !IsOriginalNode(node)) {
-    // Node is not scheduled yet, so we can add it directly.
-    return AddNode(node);
-  } else {
-    // TODO(9684): Potentially add some per-block caching so we can avoid
-    // cloning if we've already cloned for this block.
-    return AddNode(graph_->CloneNode(node));
-  }
-}
-
-bool GraphAssembler::BasicBlockUpdater::IsOriginalNode(Node* node) {
-  // Return true if node was part of the original schedule and might currently
-  // be re-added to the schedule after a CopyForChange.
-  return node->id() < original_node_count_;
-}
-
-void GraphAssembler::BasicBlockUpdater::CopyForChange() {
-  DCHECK_EQ(kUnchanged, state_);
-
-  // Save successor.
-  DCHECK(saved_successors_.empty());
-  for (BasicBlock* successor : original_block()->successors()) {
-    for (size_t i = 0; i < successor->PredecessorCount(); i++) {
-      if (successor->PredecessorAt(i) == original_block()) {
-        saved_successors_.push_back({successor, i});
-        break;
-      }
-    }
-  }
-  DCHECK_EQ(saved_successors_.size(), original_block()->SuccessorCount());
-
-  // Save control.
-  original_control_ = original_block()->control();
-  original_control_input_ = original_block()->control_input();
-
-  // Save original nodes (to allow them to continue to be iterated by the user
-  // of graph assembler).
-  original_block()->nodes()->swap(saved_nodes_);
-  DCHECK(original_block()->nodes()->empty());
-
-  // Re-insert the nodes from the front of the block.
-  original_block()->InsertNodes(original_block()->begin(), saved_nodes_.begin(),
-                                node_it_);
-
-  // Remove the tail from the schedule.
-  for (; node_it_ != end_it_; node_it_++) {
-    schedule_->SetBlockForNode(nullptr, *node_it_);
-  }
-
-  // Reset the control.
-  if (original_block()->control() != BasicBlock::kGoto) {
-    schedule_->SetBlockForNode(nullptr, original_block()->control_input());
-  }
-  original_block()->set_control_input(nullptr);
-  original_block()->set_control(BasicBlock::kNone);
-  original_block()->ClearSuccessors();
-
-  state_ = kChanged;
-  end_it_ = {};
-  node_it_ = {};
-}
-
-BasicBlock* GraphAssembler::BasicBlockUpdater::NewBasicBlock(bool deferred) {
-  BasicBlock* block = schedule_->NewBasicBlock();
-  block->set_deferred(deferred || original_deferred_);
-  return block;
-}
-
-BasicBlock* GraphAssembler::BasicBlockUpdater::SplitBasicBlock() {
-  return NewBasicBlock(current_block_->deferred());
-}
-
-void GraphAssembler::BasicBlockUpdater::AddBind(BasicBlock* to) {
-  DCHECK_NOT_NULL(to);
-  current_block_ = to;
-  // Basic block should only have the control node, if any.
-  DCHECK_LE(current_block_->NodeCount(), 1);
-  SetBlockDeferredFromPredecessors();
-}
-
-void GraphAssembler::BasicBlockUpdater::SetBlockDeferredFromPredecessors() {
-  if (!current_block_->deferred()) {
-    bool deferred = true;
-    for (BasicBlock* pred : current_block_->predecessors()) {
-      if (!pred->deferred()) {
-        deferred = false;
-        break;
-      }
-    }
-    current_block_->set_deferred(deferred);
-  }
-}
-
-void GraphAssembler::BasicBlockUpdater::AddBranch(Node* node,
-                                                  BasicBlock* tblock,
-                                                  BasicBlock* fblock) {
-  if (state_ == kUnchanged) {
-    DCHECK_EQ(current_block_, original_block());
-    CopyForChange();
-  }
-
-  DCHECK_EQ(state_, kChanged);
-  schedule_->AddBranch(current_block_, node, tblock, fblock);
-  current_block_ = nullptr;
-}
-
-void GraphAssembler::BasicBlockUpdater::AddGoto(BasicBlock* to) {
-  DCHECK_NOT_NULL(current_block_);
-  AddGoto(current_block_, to);
-}
-
-void GraphAssembler::BasicBlockUpdater::AddGoto(BasicBlock* from,
-                                                BasicBlock* to) {
-  if (state_ == kUnchanged) {
-    CopyForChange();
-  }
-
-  if (to->deferred() && !from->deferred()) {
-    // Add a new block with the correct deferred hint to avoid merges into the
-    // target block with different deferred hints.
-    // TODO(9684): Only split the current basic block if the label's target
-    // block has multiple merges.
-    BasicBlock* new_block = NewBasicBlock(to->deferred());
-    schedule_->AddGoto(from, new_block);
-    from = new_block;
-  }
-
-  schedule_->AddGoto(from, to);
-  current_block_ = nullptr;
-}
-
-void GraphAssembler::BasicBlockUpdater::AddTailCall(Node* node) {
-  DCHECK_EQ(node->opcode(), IrOpcode::kTailCall);
-  DCHECK_NOT_NULL(current_block_);
-
-  if (state_ == kUnchanged) {
-    CopyForChange();
-  }
-
-  schedule_->AddTailCall(current_block_, node);
-  current_block_ = nullptr;
-}
-
-void GraphAssembler::BasicBlockUpdater::UpdateSuccessors(BasicBlock* block) {
-  for (SuccessorInfo succ : saved_successors_) {
-    (succ.block->predecessors())[succ.index] = block;
-    block->AddSuccessor(succ.block);
-  }
-  saved_successors_.clear();
-  block->set_control(original_control_);
-  block->set_control_input(original_control_input_);
-  if (original_control_input_ != nullptr) {
-    schedule_->SetBlockForNode(block, original_control_input_);
-  } else {
-    DCHECK_EQ(BasicBlock::kGoto, original_control_);
-  }
-}
-
-void GraphAssembler::BasicBlockUpdater::StartBlock(BasicBlock* block) {
-  DCHECK_NULL(current_block_);
-  DCHECK_NULL(original_block_);
-  DCHECK(saved_nodes_.empty());
-  block->ResetRPOInfo();
-  current_block_ = block;
-  original_block_ = block;
-  original_deferred_ = block->deferred();
-  node_it_ = block->begin();
-  end_it_ = block->end();
-  state_ = kUnchanged;
-}
-
-BasicBlock* GraphAssembler::BasicBlockUpdater::Finalize(BasicBlock* original) {
-  DCHECK_EQ(original, original_block());
-  BasicBlock* block = current_block_;
-  if (state_ == kChanged) {
-    UpdateSuccessors(block);
-  } else {
-    DCHECK_EQ(block, original_block());
-    if (node_it_ != end_it_) {
-      // We have not got to the end of the node list, we need to trim.
-      block->TrimNodes(node_it_);
-    }
-  }
-  original_control_ = BasicBlock::kNone;
-  saved_nodes_.clear();
-  original_deferred_ = false;
-  original_control_input_ = nullptr;
-  original_block_ = nullptr;
-  current_block_ = nullptr;
-  return block;
-}
-
 class V8_NODISCARD GraphAssembler::BlockInlineReduction {
  public:
   explicit BlockInlineReduction(GraphAssembler* gasm) : gasm_(gasm) {
@@ -349,16 +37,12 @@ class V8_NODISCARD GraphAssembler::BlockInlineReduction {
 GraphAssembler::GraphAssembler(
     MachineGraph* mcgraph, Zone* zone,
     base::Optional<NodeChangedCallback> node_changed_callback,
-    Schedule* schedule, bool mark_loop_exits)
+    bool mark_loop_exits)
     : temp_zone_(zone),
       mcgraph_(mcgraph),
       effect_(nullptr),
       control_(nullptr),
       node_changed_callback_(node_changed_callback),
-      block_updater_(schedule != nullptr
-                         ? new BasicBlockUpdater(schedule, mcgraph->graph(),
-                                                 mcgraph->common(), zone)
-                         : nullptr),
       inline_reducers_(zone),
       inline_reductions_blocked_(false),
       loop_headers_(zone),
@@ -711,13 +395,8 @@ Node* GraphAssembler::DebugBreak() {
 Node* GraphAssembler::Unreachable(
     GraphAssemblerLabel<0u>* block_updater_successor) {
   Node* result = UnreachableWithoutConnectToEnd();
-  if (block_updater_ == nullptr) {
-    ConnectUnreachableToEnd();
-    InitializeEffectControl(nullptr, nullptr);
-  } else {
-    DCHECK_NOT_NULL(block_updater_successor);
-    Goto(block_updater_successor);
-  }
+  ConnectUnreachableToEnd();
+  InitializeEffectControl(nullptr, nullptr);
   return result;
 }
 
@@ -890,8 +569,6 @@ void GraphAssembler::TailCall(const CallDescriptor* call_descriptor,
   Node* node = AddNode(graph()->NewNode(common()->TailCall(call_descriptor),
                                         inputs_size, inputs));
 
-  if (block_updater_) block_updater_->AddTailCall(node);
-
   // Unlike ConnectUnreachableToEnd, the TailCall node terminates a block; to
   // keep it live, it *must* be connected to End (also in Turboprop schedules).
   NodeProperties::MergeControlToEnd(graph(), common(), node);
@@ -912,103 +589,18 @@ void GraphAssembler::BranchWithCriticalSafetyCheck(
   BranchImpl(condition, if_true, if_false, hint);
 }
 
-void GraphAssembler::RecordBranchInBlockUpdater(Node* branch,
-                                                Node* if_true_control,
-                                                Node* if_false_control,
-                                                BasicBlock* if_true_block,
-                                                BasicBlock* if_false_block) {
-  DCHECK_NOT_NULL(block_updater_);
-  // TODO(9684): Only split the current basic block if the label's target
-  // block has multiple merges.
-  BasicBlock* if_true_target = block_updater_->SplitBasicBlock();
-  BasicBlock* if_false_target = block_updater_->SplitBasicBlock();
-
-  block_updater_->AddBranch(branch, if_true_target, if_false_target);
-
-  block_updater_->AddNode(if_true_control, if_true_target);
-  block_updater_->AddGoto(if_true_target, if_true_block);
-
-  block_updater_->AddNode(if_false_control, if_false_target);
-  block_updater_->AddGoto(if_false_target, if_false_block);
-}
-
-void GraphAssembler::BindBasicBlock(BasicBlock* block) {
-  if (block_updater_) {
-    block_updater_->AddBind(block);
-  }
-}
-
-BasicBlock* GraphAssembler::NewBasicBlock(bool deferred) {
-  if (!block_updater_) return nullptr;
-  return block_updater_->NewBasicBlock(deferred);
-}
-
-void GraphAssembler::GotoBasicBlock(BasicBlock* block) {
-  if (block_updater_) {
-    block_updater_->AddGoto(block);
-  }
-}
-
-void GraphAssembler::GotoIfBasicBlock(BasicBlock* block, Node* branch,
-                                      IrOpcode::Value goto_if) {
-  if (block_updater_) {
-    // TODO(9684): Only split the current basic block for the goto_target
-    // if block has multiple merges.
-    BasicBlock* goto_target = block_updater_->SplitBasicBlock();
-    BasicBlock* fallthrough_target = block_updater_->SplitBasicBlock();
-
-    if (goto_if == IrOpcode::kIfTrue) {
-      block_updater_->AddBranch(branch, goto_target, fallthrough_target);
-    } else {
-      DCHECK_EQ(goto_if, IrOpcode::kIfFalse);
-      block_updater_->AddBranch(branch, fallthrough_target, goto_target);
-    }
-
-    block_updater_->AddNode(control(), goto_target);
-    block_updater_->AddGoto(goto_target, block);
-
-    block_updater_->AddBind(fallthrough_target);
-  }
-}
-
-BasicBlock* GraphAssembler::FinalizeCurrentBlock(BasicBlock* block) {
-  if (block_updater_) {
-    block = block_updater_->Finalize(block);
-    if (control() == mcgraph()->Dead()) {
-      // If the block's end is unreachable, then reset current effect and
-      // control to that of the block's throw control node.
-      DCHECK(block->control() == BasicBlock::kThrow);
-      Node* throw_node = block->control_input();
-      control_ = NodeProperties::GetControlInput(throw_node);
-      effect_ = NodeProperties::GetEffectInput(throw_node);
-    }
-  }
-  return block;
-}
-
 void GraphAssembler::ConnectUnreachableToEnd() {
   DCHECK_EQ(effect()->opcode(), IrOpcode::kUnreachable);
-  // When maintaining the schedule we can't easily rewire the successor blocks
-  // to disconnect them from the graph, so we just leave the unreachable nodes
-  // in the schedule.
-  // TODO(9684): Add a scheduled dead-code elimination phase to remove all the
-  // subsequent unreachable code from the schedule.
-  if (!block_updater_) {
-    Node* throw_node = graph()->NewNode(common()->Throw(), effect(), control());
-    NodeProperties::MergeControlToEnd(graph(), common(), throw_node);
-    if (node_changed_callback_.has_value()) {
-      (*node_changed_callback_)(graph()->end());
-    }
-    effect_ = control_ = mcgraph()->Dead();
+  Node* throw_node = graph()->NewNode(common()->Throw(), effect(), control());
+  NodeProperties::MergeControlToEnd(graph(), common(), throw_node);
+  if (node_changed_callback_.has_value()) {
+    (*node_changed_callback_)(graph()->end());
   }
+  effect_ = control_ = mcgraph()->Dead();
 }
 
 Node* GraphAssembler::AddClonedNode(Node* node) {
   DCHECK(node->op()->HasProperty(Operator::kPure));
-  if (block_updater_) {
-    node = block_updater_->AddClonedNode(node);
-  }
-
   UpdateEffectControlWith(node);
   return node;
 }
@@ -1036,10 +628,6 @@ Node* GraphAssembler::AddNode(Node* node) {
     }
   }
 
-  if (block_updater_) {
-    block_updater_->AddNode(node);
-  }
-
   if (node->opcode() == IrOpcode::kTerminate) {
     return node;
   }
@@ -1048,12 +636,9 @@ Node* GraphAssembler::AddNode(Node* node) {
   return node;
 }
 
-void GraphAssembler::Reset(BasicBlock* block) {
+void GraphAssembler::Reset() {
   effect_ = nullptr;
   control_ = nullptr;
-  if (block_updater_) {
-    block_updater_->StartBlock(block);
-  }
 }
 
 void GraphAssembler::InitializeEffectControl(Node* effect, Node* control) {
