@@ -563,8 +563,7 @@ int InstanceSizeWithMinSlack(JSHeapBroker* broker, MapRef map) {
     DCHECK(map.object()->GetBackPointer().IsUndefined(broker->isolate()));
 
     static constexpr bool kConcurrentAccess = true;
-    TransitionsAccessor(broker->isolate(), *map.object(), &no_gc,
-                        kConcurrentAccess)
+    TransitionsAccessor(broker->isolate(), *map.object(), kConcurrentAccess)
         .TraverseTransitionTree([&](Map m) {
           maps.push_back(broker->CanonicalPersistentHandle(m));
         });
@@ -1572,10 +1571,8 @@ BROKER_SFI_FIELDS(DEF_SFI_ACCESSOR)
 SharedFunctionInfo::Inlineability SharedFunctionInfoRef::GetInlineability()
     const {
   return broker()->IsMainThread()
-             ? object()->GetInlineability(broker()->isolate(),
-                                          broker()->is_turboprop())
-             : object()->GetInlineability(broker()->local_isolate(),
-                                          broker()->is_turboprop());
+             ? object()->GetInlineability(broker()->isolate())
+             : object()->GetInlineability(broker()->local_isolate());
 }
 
 ObjectRef FeedbackCellRef::value() const {
@@ -2167,8 +2164,10 @@ BIMODAL_ACCESSOR(JSFunction, SharedFunctionInfo, shared)
 #undef JSFUNCTION_BIMODAL_ACCESSOR_WITH_DEP_C
 
 CodeRef JSFunctionRef::code() const {
-  return MakeRefAssumeMemoryFence(broker(),
-                                  FromCodeT(object()->code(kAcquireLoad)));
+  CodeT code = object()->code(kAcquireLoad);
+  // Safe to do a relaxed conversion to Code here since CodeT::code field is
+  // modified only by GC and the CodeT was acquire-loaded.
+  return MakeRefAssumeMemoryFence(broker(), FromCodeT(code, kRelaxedLoad));
 }
 
 NativeContextRef JSFunctionRef::native_context() const {
