@@ -40,7 +40,7 @@ bool Stack::IsOnStack(void* slot) const {
 #if defined(__has_feature)
 #if __has_feature(safe_stack)
   if (__builtin___get_unsafe_stack_top() >= slot &&
-      slot > __builtin___get_unsafe_stack_ptr()) {
+      slot >= __builtin___get_unsafe_stack_ptr()) {
     return true;
   }
 #endif  // __has_feature(safe_stack)
