@@ -463,7 +463,8 @@ class CppGraphBuilderImpl final {
   void AddEdge(State& parent, const TracedReferenceBase& ref,
                const std::string& edge_name) {
     DCHECK(parent.IsVisibleNotDependent());
-    v8::Local<v8::Value> v8_value = ref.Get(cpp_heap_.isolate());
+    v8::Local<v8::Value> v8_value =
+        ref.Get(reinterpret_cast<v8::Isolate*>(cpp_heap_.isolate()));
     if (!v8_value.IsEmpty()) {
       if (!parent.get_node()) {
         parent.set_node(AddNode(*parent.header()));
@@ -836,7 +837,8 @@ void CppGraphBuilderImpl::VisitWeakContainerForVisibility(
 
 void CppGraphBuilderImpl::VisitForVisibility(State& parent,
                                              const TracedReferenceBase& ref) {
-  v8::Local<v8::Value> v8_value = ref.Get(cpp_heap_.isolate());
+  v8::Local<v8::Value> v8_value =
+      ref.Get(reinterpret_cast<v8::Isolate*>(cpp_heap_.isolate()));
   if (!v8_value.IsEmpty()) {
     parent.MarkVisible();
   }
