@@ -66,26 +66,6 @@ T ForwardingAddress(T heap_obj) {
   }
 }
 
-AllocationSpace AllocationResult::RetrySpace() {
-  DCHECK(IsRetry());
-  return static_cast<AllocationSpace>(Smi::ToInt(object_));
-}
-
-HeapObject AllocationResult::ToObjectChecked() {
-  CHECK(!IsRetry());
-  return HeapObject::cast(object_);
-}
-
-HeapObject AllocationResult::ToObject() {
-  DCHECK(!IsRetry());
-  return HeapObject::cast(object_);
-}
-
-Address AllocationResult::ToAddress() {
-  DCHECK(!IsRetry());
-  return HeapObject::cast(object_).address();
-}
-
 // static
 base::EnumSet<CodeFlushMode> Heap::GetCodeFlushMode(Isolate* isolate) {
   if (isolate->disable_bytecode_flushing()) {
@@ -191,6 +171,10 @@ inline const base::AddressRegion& Heap::code_region() {
 #endif
 }
 
+Address Heap::code_range_base() {
+  return code_range_ ? code_range_->base() : kNullAddress;
+}
+
 int Heap::MaxRegularHeapObjectSize(AllocationType allocation) {
   if (!V8_ENABLE_THIRD_PARTY_HEAP_BOOL &&
       (allocation == AllocationType::kCode)) {
@@ -204,29 +188,27 @@ int Heap::MaxRegularHeapObjectSize(AllocationType allocation) {
 AllocationResult Heap::AllocateRaw(int size_in_bytes, AllocationType type,
                                    AllocationOrigin origin,
                                    AllocationAlignment alignment) {
+  DCHECK_EQ(gc_state(), NOT_IN_GC);
   DCHECK(AllowHandleAllocation::IsAllowed());
   DCHECK(AllowHeapAllocation::IsAllowed());
-  DCHECK_IMPLIES(type == AllocationType::kCode || type == AllocationType::kMap,
-                 alignment == AllocationAlignment::kTaggedAligned);
-  DCHECK_EQ(gc_state(), NOT_IN_GC);
 #ifdef V8_ENABLE_ALLOCATION_TIMEOUT
   if (FLAG_random_gc_interval > 0 || FLAG_gc_interval >= 0) {
     if (!always_allocate() && Heap::allocation_timeout_-- <= 0) {
       AllocationSpace space = FLAG_single_generation ? OLD_SPACE : NEW_SPACE;
-      return AllocationResult::Retry(space);
+      return AllocationResult::Failure(space);
     }
   }
-#endif
+#endif  // V8_ENABLE_ALLOCATION_TIMEOUT
 #ifdef DEBUG
   IncrementObjectCounters();
-#endif
+#endif  // DEBUG
 
   if (CanSafepoint()) {
     main_thread_local_heap()->Safepoint();
   }
 
-  size_t large_object_threshold = MaxRegularHeapObjectSize(type);
-  bool large_object =
+  const size_t large_object_threshold = MaxRegularHeapObjectSize(type);
+  const bool large_object =
       static_cast<size_t>(size_in_bytes) > large_object_threshold;
 
   HeapObject object;
@@ -239,48 +221,42 @@ AllocationResult Heap::AllocateRaw(int size_in_bytes, AllocationType type,
   if (V8_ENABLE_THIRD_PARTY_HEAP_BOOL) {
     allocation = tp_heap_->Allocate(size_in_bytes, type, alignment);
   } else {
-    if (AllocationType::kYoung == type) {
-      if (large_object) {
-        if (FLAG_young_generation_large_objects) {
-          allocation = new_lo_space_->AllocateRaw(size_in_bytes);
-        } else {
-          // If young generation large objects are disalbed we have to tenure
-          // the allocation and violate the given allocation type. This could be
-          // dangerous. We may want to remove
-          // FLAG_young_generation_large_objects and avoid patching.
-          allocation = lo_space_->AllocateRaw(size_in_bytes);
-        }
-      } else {
-        allocation = new_space_->AllocateRaw(size_in_bytes, alignment, origin);
-      }
-    } else if (AllocationType::kOld == type) {
-      if (large_object) {
-        allocation = lo_space_->AllocateRaw(size_in_bytes);
-      } else {
-        allocation = old_space_->AllocateRaw(size_in_bytes, alignment, origin);
-      }
-    } else if (AllocationType::kCode == type) {
-      DCHECK(AllowCodeAllocation::IsAllowed());
-      if (large_object) {
-        allocation = code_lo_space_->AllocateRaw(size_in_bytes);
-      } else {
-        allocation = code_space_->AllocateRawUnaligned(size_in_bytes);
-      }
-    } else if (AllocationType::kMap == type) {
-      allocation = map_space_->AllocateRawUnaligned(size_in_bytes);
-    } else if (AllocationType::kReadOnly == type) {
-      DCHECK(!large_object);
-      DCHECK(CanAllocateInReadOnlySpace());
-      DCHECK_EQ(AllocationOrigin::kRuntime, origin);
-      allocation = read_only_space_->AllocateRaw(size_in_bytes, alignment);
-    } else if (AllocationType::kSharedOld == type) {
+    if (V8_UNLIKELY(large_object)) {
       allocation =
-          shared_old_allocator_->AllocateRaw(size_in_bytes, alignment, origin);
-    } else if (AllocationType::kSharedMap == type) {
-      allocation =
-          shared_map_allocator_->AllocateRaw(size_in_bytes, alignment, origin);
+          AllocateRawLargeInternal(size_in_bytes, type, origin, alignment);
     } else {
-      UNREACHABLE();
+      switch (type) {
+        case AllocationType::kYoung:
+          allocation =
+              new_space_->AllocateRaw(size_in_bytes, alignment, origin);
+          break;
+        case AllocationType::kOld:
+          allocation =
+              old_space_->AllocateRaw(size_in_bytes, alignment, origin);
+          break;
+        case AllocationType::kCode:
+          DCHECK_EQ(alignment, AllocationAlignment::kTaggedAligned);
+          DCHECK(AllowCodeAllocation::IsAllowed());
+          allocation = code_space_->AllocateRawUnaligned(size_in_bytes);
+          break;
+        case AllocationType::kMap:
+          DCHECK_EQ(alignment, AllocationAlignment::kTaggedAligned);
+          allocation = map_space_->AllocateRawUnaligned(size_in_bytes);
+          break;
+        case AllocationType::kReadOnly:
+          DCHECK(CanAllocateInReadOnlySpace());
+          DCHECK_EQ(AllocationOrigin::kRuntime, origin);
+          allocation = read_only_space_->AllocateRaw(size_in_bytes, alignment);
+          break;
+        case AllocationType::kSharedMap:
+          allocation = shared_map_allocator_->AllocateRaw(size_in_bytes,
+                                                          alignment, origin);
+          break;
+        case AllocationType::kSharedOld:
+          allocation = shared_old_allocator_->AllocateRaw(size_in_bytes,
+                                                          alignment, origin);
+          break;
+      }
     }
   }
 
@@ -304,9 +280,11 @@ AllocationResult Heap::AllocateRaw(int size_in_bytes, AllocationType type,
       Page::FromHeapObject(object)->object_start_bitmap()->SetBit(
           object.address());
     }
-#endif
+#endif  // V8_ENABLE_CONSERVATIVE_STACK_SCANNING
 
-    OnAllocationEvent(object, size_in_bytes);
+    for (auto& tracker : allocation_trackers_) {
+      tracker->AllocationEvent(object.address(), size_in_bytes);
+    }
   }
 
   return allocation;
@@ -319,21 +297,15 @@ HeapObject Heap::AllocateRawWith(int size, AllocationType allocation,
   DCHECK(AllowHandleAllocation::IsAllowed());
   DCHECK(AllowHeapAllocation::IsAllowed());
   DCHECK_EQ(gc_state(), NOT_IN_GC);
-  Heap* heap = isolate()->heap();
-  if (allocation == AllocationType::kYoung &&
-      alignment == AllocationAlignment::kTaggedAligned &&
-      size <= MaxRegularHeapObjectSize(allocation) &&
-      V8_LIKELY(!FLAG_single_generation && FLAG_inline_new &&
-                FLAG_gc_interval == -1)) {
-    Address* top = heap->NewSpaceAllocationTopAddress();
-    Address* limit = heap->NewSpaceAllocationLimitAddress();
-    if (*limit - *top >= static_cast<unsigned>(size)) {
-      DCHECK(IsAligned(size, kTaggedSize));
-      HeapObject obj = HeapObject::FromAddress(*top);
-      *top += size;
-      MSAN_ALLOCATED_UNINITIALIZED_MEMORY(obj.address(), size);
-      return obj;
-    }
+  if (allocation == AllocationType::kYoung) {
+    auto result = AllocateRaw(size, AllocationType::kYoung, origin, alignment);
+    HeapObject object;
+    if (result.To(&object)) return object;
+
+  } else if (allocation == AllocationType::kOld) {
+    auto result = AllocateRaw(size, AllocationType::kOld, origin, alignment);
+    HeapObject object;
+    if (result.To(&object)) return object;
   }
   switch (mode) {
     case kLightRetry:
@@ -353,58 +325,10 @@ Address Heap::AllocateRawOrFail(int size, AllocationType allocation,
       .address();
 }
 
-void Heap::OnAllocationEvent(HeapObject object, int size_in_bytes) {
-  for (auto& tracker : allocation_trackers_) {
-    tracker->AllocationEvent(object.address(), size_in_bytes);
-  }
-
-  if (FLAG_verify_predictable) {
-    ++allocations_count_;
-    // Advance synthetic time by making a time request.
-    MonotonicallyIncreasingTimeInMs();
-
-    UpdateAllocationsHash(object);
-    UpdateAllocationsHash(size_in_bytes);
-
-    if (allocations_count_ % FLAG_dump_allocations_digest_at_alloc == 0) {
-      PrintAllocationsHash();
-    }
-  } else if (FLAG_fuzzer_gc_analysis) {
-    ++allocations_count_;
-  } else if (FLAG_trace_allocation_stack_interval > 0) {
-    ++allocations_count_;
-    if (allocations_count_ % FLAG_trace_allocation_stack_interval == 0) {
-      isolate()->PrintStack(stdout, Isolate::kPrintStackConcise);
-    }
-  }
-}
-
 bool Heap::CanAllocateInReadOnlySpace() {
   return read_only_space()->writable();
 }
 
-void Heap::UpdateAllocationsHash(HeapObject object) {
-  Address object_address = object.address();
-  MemoryChunk* memory_chunk = MemoryChunk::FromAddress(object_address);
-  AllocationSpace allocation_space = memory_chunk->owner_identity();
-
-  STATIC_ASSERT(kSpaceTagSize + kPageSizeBits <= 32);
-  uint32_t value =
-      static_cast<uint32_t>(object_address - memory_chunk->address()) |
-      (static_cast<uint32_t>(allocation_space) << kPageSizeBits);
-
-  UpdateAllocationsHash(value);
-}
-
-void Heap::UpdateAllocationsHash(uint32_t value) {
-  uint16_t c1 = static_cast<uint16_t>(value);
-  uint16_t c2 = static_cast<uint16_t>(value >> 16);
-  raw_allocations_hash_ =
-      StringHasher::AddCharacterCore(raw_allocations_hash_, c1);
-  raw_allocations_hash_ =
-      StringHasher::AddCharacterCore(raw_allocations_hash_, c2);
-}
-
 void Heap::RegisterExternalString(String string) {
   DCHECK(string.IsExternalString());
   DCHECK(!string.IsThinString());
