@@ -121,15 +121,16 @@ void MarkingBarrier::Write(DescriptorArray descriptor_array,
 void MarkingBarrier::RecordRelocSlot(Code host, RelocInfo* rinfo,
                                      HeapObject target) {
   DCHECK(IsCurrentMarkingBarrier());
+  if (!MarkCompactCollector::ShouldRecordRelocSlot(host, rinfo, target)) return;
+
   MarkCompactCollector::RecordRelocSlotInfo info =
-      MarkCompactCollector::PrepareRecordRelocSlot(host, rinfo, target);
-  if (info.should_record) {
-    auto& typed_slots = typed_slots_map_[info.memory_chunk];
-    if (!typed_slots) {
-      typed_slots.reset(new TypedSlots());
-    }
-    typed_slots->Insert(info.slot_type, info.offset);
+      MarkCompactCollector::ProcessRelocInfo(host, rinfo, target);
+
+  auto& typed_slots = typed_slots_map_[info.memory_chunk];
+  if (!typed_slots) {
+    typed_slots.reset(new TypedSlots());
   }
+  typed_slots->Insert(info.slot_type, info.offset);
 }
 
 // static
