@@ -10,7 +10,7 @@
 #include "src/execution/isolate.h"
 #include "src/handles/global-handles.h"
 #include "src/logging/counters.h"
-#include "src/security/vm-cage.h"
+#include "src/sandbox/sandbox.h"
 
 #if V8_ENABLE_WEBASSEMBLY
 #include "src/trap-handler/trap-handler.h"
@@ -55,13 +55,15 @@ enum class AllocationStatus {
   kOtherFailure  // Failed for an unknown reason
 };
 
-// Attempts to allocate memory inside the virtual memory cage currently fall
-// back to allocating memory outside of the cage if necessary. Once this
-// fallback is no longer allowed/possible, these cases will become allocation
-// failures instead. To track the frequency of such events, the outcome of
-// memory allocation attempts inside the cage is reported to UMA.
+// Attempts to allocate memory inside the sandbox currently fall back to
+// allocating memory outside of the sandbox if necessary. Once this fallback is
+// no longer allowed/possible, these cases will become allocation failures
+// instead. To track the frequency of such events, the outcome of memory
+// allocation attempts inside the sandbox is reported to UMA.
 //
 // See caged_memory_allocation_outcome in counters-definitions.h
+// This class and the entry in counters-definitions.h use the term "cage"
+// instead of "sandbox" for historical reasons.
 enum class CagedMemoryAllocationOutcome {
   kSuccess,      // Allocation succeeded inside the cage
   kOutsideCage,  // Allocation failed inside the cage but succeeded outside
@@ -107,18 +109,17 @@ void RecordStatus(Isolate* isolate, AllocationStatus status) {
       static_cast<int>(status));
 }
 
-// When the virtual memory cage is active, this function records the outcome of
-// attempts to allocate memory inside the cage which fall back to allocating
-// memory outside of the cage. Passing a value of nullptr for the result
-// indicates that the memory could not be allocated at all.
-void RecordCagedMemoryAllocationResult(Isolate* isolate, void* result) {
-  // This metric is only meaningful when the virtual memory cage is active.
-#ifdef V8_VIRTUAL_MEMORY_CAGE
-  if (GetProcessWideVirtualMemoryCage()->is_initialized()) {
+// When the sandbox is active, this function records the outcome of attempts to
+// allocate memory inside the sandbox which fall back to allocating memory
+// outside of the sandbox. Passing a value of nullptr for the result indicates
+// that the memory could not be allocated at all.
+void RecordSandboxMemoryAllocationResult(Isolate* isolate, void* result) {
+  // This metric is only meaningful when the sandbox is active.
+#ifdef V8_SANDBOX
+  if (GetProcessWideSandbox()->is_initialized()) {
     CagedMemoryAllocationOutcome outcome;
     if (result) {
-      bool allocation_in_cage =
-          GetProcessWideVirtualMemoryCage()->Contains(result);
+      bool allocation_in_cage = GetProcessWideSandbox()->Contains(result);
       outcome = allocation_in_cage ? CagedMemoryAllocationOutcome::kSuccess
                                    : CagedMemoryAllocationOutcome::kOutsideCage;
     } else {
@@ -210,11 +211,11 @@ BackingStore::~BackingStore() {
   // TODO(saelo) here and elsewhere in this file, replace with
   // GetArrayBufferPageAllocator once the fallback to the platform page
   // allocator is no longer allowed.
-#ifdef V8_VIRTUAL_MEMORY_CAGE
-  if (GetProcessWideVirtualMemoryCage()->Contains(buffer_start_)) {
-    page_allocator = GetVirtualMemoryCagePageAllocator();
+#ifdef V8_SANDBOX
+  if (GetProcessWideSandbox()->Contains(buffer_start_)) {
+    page_allocator = GetSandboxPageAllocator();
   } else {
-    DCHECK(kAllowBackingStoresOutsideCage);
+    DCHECK(kAllowBackingStoresOutsideSandbox);
   }
 #endif
 
@@ -327,7 +328,6 @@ std::unique_ptr<BackingStore> BackingStore::Allocate(
     }
   }
 
-  DCHECK(IsValidBackingStorePointer(buffer_start));
   auto result = new BackingStore(buffer_start,                  // start
                                  byte_length,                   // length
                                  byte_length,                   // max length
@@ -428,18 +428,18 @@ std::unique_ptr<BackingStore> BackingStore::TryAllocateAndPartiallyCommitMemory(
   void* allocation_base = nullptr;
   PageAllocator* page_allocator = GetPlatformPageAllocator();
   auto allocate_pages = [&] {
-#ifdef V8_VIRTUAL_MEMORY_CAGE
-    page_allocator = GetVirtualMemoryCagePageAllocator();
+#ifdef V8_SANDBOX
+    page_allocator = GetSandboxPageAllocator();
     allocation_base = AllocatePages(page_allocator, nullptr, reservation_size,
                                     page_size, PageAllocator::kNoAccess);
     if (allocation_base) return true;
     // We currently still allow falling back to the platform page allocator if
-    // the cage page allocator fails. This will eventually be removed.
+    // the sandbox page allocator fails. This will eventually be removed.
     // TODO(chromium:1218005) once we forbid the fallback, we should have a
     // single API, e.g. GetArrayBufferPageAllocator(), that returns the correct
-    // page allocator to use here depending on whether the virtual memory cage
-    // is enabled or not.
-    if (!kAllowBackingStoresOutsideCage) return false;
+    // page allocator to use here depending on whether the sandbox is enabled
+    // or not.
+    if (!kAllowBackingStoresOutsideSandbox) return false;
     page_allocator = GetPlatformPageAllocator();
 #endif
     allocation_base = AllocatePages(page_allocator, nullptr, reservation_size,
@@ -449,13 +449,11 @@ std::unique_ptr<BackingStore> BackingStore::TryAllocateAndPartiallyCommitMemory(
   if (!gc_retry(allocate_pages)) {
     // Page allocator could not reserve enough pages.
     RecordStatus(isolate, AllocationStatus::kOtherFailure);
-    RecordCagedMemoryAllocationResult(isolate, nullptr);
+    RecordSandboxMemoryAllocationResult(isolate, nullptr);
     TRACE_BS("BSw:try   failed to allocate pages\n");
     return {};
   }
 
-  DCHECK(IsValidBackingStorePointer(allocation_base));
-
   // Get a pointer to the start of the buffer, skipping negative guard region
   // if necessary.
 #if V8_ENABLE_WEBASSEMBLY
@@ -478,15 +476,17 @@ std::unique_ptr<BackingStore> BackingStore::TryAllocateAndPartiallyCommitMemory(
   if (!gc_retry(commit_memory)) {
     TRACE_BS("BSw:try   failed to set permissions (%p, %zu)\n", buffer_start,
              committed_byte_length);
+    CHECK(FreePages(page_allocator, allocation_base, reservation_size));
     // SetPermissions put us over the process memory limit.
-    V8::FatalProcessOutOfMemory(nullptr, "BackingStore::AllocateMemory()");
+    // We return an empty result so that the caller can throw an exception.
+    return {};
   }
 
   DebugCheckZero(buffer_start, byte_length);  // touch the bytes.
 
   RecordStatus(isolate, did_retry ? AllocationStatus::kSuccessAfterRetry
                                   : AllocationStatus::kSuccess);
-  RecordCagedMemoryAllocationResult(isolate, allocation_base);
+  RecordSandboxMemoryAllocationResult(isolate, allocation_base);
 
   ResizableFlag resizable =
       is_wasm_memory ? ResizableFlag::kNotResizable : ResizableFlag::kResizable;
@@ -752,7 +752,6 @@ BackingStore::ResizeOrGrowResult BackingStore::GrowInPlace(
 std::unique_ptr<BackingStore> BackingStore::WrapAllocation(
     Isolate* isolate, void* allocation_base, size_t allocation_length,
     SharedFlag shared, bool free_on_destruct) {
-  DCHECK(IsValidBackingStorePointer(allocation_base));
   auto result = new BackingStore(allocation_base,               // start
                                  allocation_length,             // length
                                  allocation_length,             // max length
@@ -774,7 +773,6 @@ std::unique_ptr<BackingStore> BackingStore::WrapAllocation(
     void* allocation_base, size_t allocation_length,
     v8::BackingStore::DeleterCallback deleter, void* deleter_data,
     SharedFlag shared) {
-  DCHECK(IsValidBackingStorePointer(allocation_base));
   bool is_empty_deleter = (deleter == v8::BackingStore::EmptyDeleter);
   auto result = new BackingStore(allocation_base,               // start
                                  allocation_length,             // length
