@@ -87,9 +87,6 @@ V8_WARN_UNUSED_RESULT bool HighestTierOf(CodeKinds kinds,
   if ((kinds & CodeKindFlag::TURBOFAN) != 0) {
     *highest_tier = CodeKind::TURBOFAN;
     return true;
-  } else if ((kinds & CodeKindFlag::TURBOPROP) != 0) {
-    *highest_tier = CodeKind::TURBOPROP;
-    return true;
   } else if ((kinds & CodeKindFlag::BASELINE) != 0) {
     *highest_tier = CodeKind::BASELINE;
     return true;
@@ -120,7 +117,6 @@ base::Optional<CodeKind> JSFunction::GetActiveTier() const {
 #ifdef DEBUG
   CHECK(highest_tier == CodeKind::TURBOFAN ||
         highest_tier == CodeKind::BASELINE ||
-        highest_tier == CodeKind::TURBOPROP ||
         highest_tier == CodeKind::INTERPRETED_FUNCTION);
 
   if (highest_tier == CodeKind::INTERPRETED_FUNCTION) {
@@ -128,7 +124,7 @@ base::Optional<CodeKind> JSFunction::GetActiveTier() const {
           (CodeKindIsOptimizedJSFunction(code().kind()) &&
            code().marked_for_deoptimization()) ||
           (code().builtin_id() == Builtin::kCompileLazy &&
-           shared().IsInterpreted()));
+           shared().HasBytecodeArray() && !shared().HasBaselineCode()));
   }
 #endif  // DEBUG
 
@@ -147,24 +143,7 @@ bool JSFunction::ActiveTierIsBaseline() const {
   return GetActiveTier() == CodeKind::BASELINE;
 }
 
-bool JSFunction::ActiveTierIsToptierTurboprop() const {
-  return FLAG_turboprop_as_toptier && GetActiveTier() == CodeKind::TURBOPROP;
-}
-
-bool JSFunction::ActiveTierIsMidtierTurboprop() const {
-  return FLAG_turboprop && !FLAG_turboprop_as_toptier &&
-         GetActiveTier() == CodeKind::TURBOPROP;
-}
-
-CodeKind JSFunction::NextTier() const {
-  if (V8_UNLIKELY(FLAG_turboprop) && ActiveTierIsMidtierTurboprop()) {
-    return CodeKind::TURBOFAN;
-  } else if (V8_UNLIKELY(FLAG_turboprop)) {
-    DCHECK(ActiveTierIsIgnition() || ActiveTierIsBaseline());
-    return CodeKind::TURBOPROP;
-  }
-  return CodeKind::TURBOFAN;
-}
+CodeKind JSFunction::NextTier() const { return CodeKind::TURBOFAN; }
 
 bool JSFunction::CanDiscardCompiled() const {
   // Essentially, what we are asking here is, has this function been compiled
@@ -261,18 +240,16 @@ void JSFunction::EnsureClosureFeedbackCellArray(
   Handle<SharedFunctionInfo> shared(function->shared(), isolate);
   DCHECK(function->shared().HasBytecodeArray());
 
-  bool has_closure_feedback_cell_array =
+  const bool has_closure_feedback_cell_array =
       (function->has_closure_feedback_cell_array() ||
        function->has_feedback_vector());
   // Initialize the interrupt budget to the feedback vector allocation budget
   // when initializing the feedback cell for the first time or after a bytecode
   // flush. We retain the closure feedback cell array on bytecode flush, so
   // reset_budget_for_feedback_allocation is used to reset the budget in these
-  // cases. When using a fixed allocation budget, we reset it on a bytecode
-  // flush so no additional initialization is required here.
-  if (V8_UNLIKELY(FLAG_feedback_allocation_on_bytecode_size) &&
-      (reset_budget_for_feedback_allocation ||
-       !has_closure_feedback_cell_array)) {
+  // cases.
+  if (reset_budget_for_feedback_allocation ||
+      !has_closure_feedback_cell_array) {
     function->SetInterruptBudget();
   }
 
@@ -591,6 +568,7 @@ bool CanSubclassHaveInobjectProperties(InstanceType instance_type) {
     case JS_PROMISE_TYPE:
     case JS_REG_EXP_TYPE:
     case JS_SET_TYPE:
+    case JS_SHADOW_REALM_TYPE:
     case JS_SPECIAL_API_OBJECT_TYPE:
     case JS_TYPED_ARRAY_TYPE:
     case JS_PRIMITIVE_WRAPPER_TYPE:
