@@ -114,6 +114,11 @@ class JSTemporalPlainDate
       Handle<Object> iso_month, Handle<Object> iso_day,
       Handle<Object> calendar_like);
 
+  // #sec-temporal.plaindate.prototype.withcalendar
+  V8_WARN_UNUSED_RESULT static MaybeHandle<JSTemporalPlainDate> WithCalendar(
+      Isolate* isolate, Handle<JSTemporalPlainDate> plain_date,
+      Handle<Object> calendar_like);
+
   // #sec-temporal.plaindate.prototype.getisofields
   V8_WARN_UNUSED_RESULT static MaybeHandle<JSReceiver> GetISOFields(
       Isolate* isolate, Handle<JSTemporalPlainDate> plain_date);
@@ -140,6 +145,11 @@ class JSTemporalPlainDateTime
       Handle<Object> microsecond, Handle<Object> nanosecond,
       Handle<Object> calendar_like);
 
+  // #sec-temporal.plaindatetime.prototype.withcalendar
+  V8_WARN_UNUSED_RESULT static MaybeHandle<JSTemporalPlainDateTime>
+  WithCalendar(Isolate* isolate, Handle<JSTemporalPlainDateTime> date_time,
+               Handle<Object> calendar_like);
+
   // #sec-temporal.plaindatetime.prototype.getisofields
   V8_WARN_UNUSED_RESULT static MaybeHandle<JSReceiver> GetISOFields(
       Isolate* isolate, Handle<JSTemporalPlainDateTime> date_time);
@@ -273,6 +283,18 @@ class JSTemporalZonedDateTime
       Handle<HeapObject> new_target, Handle<Object> epoch_nanoseconds,
       Handle<Object> time_zone_like, Handle<Object> calendar_like);
 
+  // #sec-temporal.zoneddatetime.prototype.withcalendar
+  V8_WARN_UNUSED_RESULT static MaybeHandle<JSTemporalZonedDateTime>
+  WithCalendar(Isolate* isolate,
+               Handle<JSTemporalZonedDateTime> zoned_date_time,
+               Handle<Object> calendar_like);
+
+  // #sec-temporal.zoneddatetime.prototype.withtimezone
+  V8_WARN_UNUSED_RESULT static MaybeHandle<JSTemporalZonedDateTime>
+  WithTimeZone(Isolate* isolate,
+               Handle<JSTemporalZonedDateTime> zoned_date_time,
+               Handle<Object> time_zone_like);
+
   // #sec-temporal.zoneddatetime.prototype.getisofields
   V8_WARN_UNUSED_RESULT static MaybeHandle<JSReceiver> GetISOFields(
       Isolate* isolate, Handle<JSTemporalZonedDateTime> zoned_date_time);
@@ -291,6 +313,23 @@ V8_WARN_UNUSED_RESULT MaybeHandle<JSTemporalInstant> CreateTemporalInstant(
 V8_WARN_UNUSED_RESULT MaybeHandle<JSTemporalInstant> CreateTemporalInstant(
     Isolate* isolate, Handle<BigInt> epoch_nanoseconds);
 
+// #sec-temporal-calendaryear
+#define DECLARE_CALENDAR_ABSTRACT_OPERATION(Name)           \
+  V8_WARN_UNUSED_RESULT MaybeHandle<Object> Calendar##Name( \
+      Isolate* isolate, Handle<JSReceiver> calendar,        \
+      Handle<JSReceiver> date_like);
+DECLARE_CALENDAR_ABSTRACT_OPERATION(Year)
+DECLARE_CALENDAR_ABSTRACT_OPERATION(Month)
+DECLARE_CALENDAR_ABSTRACT_OPERATION(MonthCode)
+DECLARE_CALENDAR_ABSTRACT_OPERATION(Day)
+
+#ifdef V8_INTL_SUPPORT
+DECLARE_CALENDAR_ABSTRACT_OPERATION(Era)
+DECLARE_CALENDAR_ABSTRACT_OPERATION(EraYear)
+#endif  //  V8_INTL_SUPPORT
+
+#undef DECLARE_CALENDAR_ABSTRACT_OPERATION
+
 // #sec-temporal-getiso8601calendar
 V8_WARN_UNUSED_RESULT MaybeHandle<JSTemporalCalendar> GetISO8601Calendar(
     Isolate* isolate);
@@ -303,6 +342,18 @@ BuiltinTimeZoneGetPlainDateTimeFor(Isolate* isolate,
                                    Handle<JSReceiver> calendar,
                                    const char* method);
 
+V8_WARN_UNUSED_RESULT MaybeHandle<Object> InvokeCalendarMethod(
+    Isolate* isolate, Handle<JSReceiver> calendar, Handle<String> name,
+    Handle<JSReceiver> temporal_like);
+
+V8_WARN_UNUSED_RESULT MaybeHandle<JSReceiver> ToTemporalCalendar(
+    Isolate* isolate, Handle<Object> temporal_calendar_like,
+    const char* method);
+
+V8_WARN_UNUSED_RESULT MaybeHandle<JSReceiver> ToTemporalTimeZone(
+    Isolate* isolate, Handle<Object> temporal_time_zone_like,
+    const char* method);
+
 }  // namespace temporal
 }  // namespace internal
 }  // namespace v8
