@@ -42,6 +42,7 @@ class CodeDataContainer;
   V(kWrapperTracing, "(Wrapper tracing)")               \
   V(kWriteBarrier, "(Write barrier)")                   \
   V(kRetainMaps, "(Retain maps)")                       \
+  V(kClientHeap, "(Client heap)")                       \
   V(kUnknown, "(Unknown)")
 
 class VisitorSynchronization : public AllStatic {
@@ -180,6 +181,12 @@ class ObjectVisitor {
 
   // Visits the object's map pointer, decoding as necessary
   virtual void VisitMapPointer(HeapObject host) { UNREACHABLE(); }
+
+  // Visits an external pointer.
+  virtual void VisitExternalPointer(HeapObject host, ExternalPointer_t ptr) {}
+
+  // Visits an EmbedderDataslot.
+  virtual void VisitEmbedderDataSlot(HeapObject host, EmbedderDataSlot slot) {}
 };
 
 // Helper version of ObjectVisitor that also takes care of caching base values
